@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-	Copyright (C) 2008-2023 Zoltan Turanyi
+	Copyright (C) 2008-2024 Zoltan Turanyi
 	Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -58,7 +58,7 @@ protected:
 public:
     static const std::vector<std::string> spec_block_names;
     using SpecBlockPtrs = std::array<const BlockBlock*, 5>;
-    bool valid = true;              ///<Set to false, if constructor fails, the arrow endpoints dont exist or its path cannot be found. Drop this instruction in PostParseProcess()
+    bool valid = true;              ///<Set to false, if constructor fails, the arrow endpoints don't exist or its path cannot be found. Drop this instruction in PostParseProcess()
     BlockChart    &chart;           ///<The block chart this element belongs to.
     const std::string name_prefix;  ///<Our prefix (without the dot): the full name of our parent (may be equal to the full name of its parent if our parent has no name). Empty only at top level.
     BlockStyle     style;           ///<The style of this element
@@ -68,8 +68,9 @@ public:
     NumberingStyle numberingStyle;  ///<The numbering style to use. (This is not part of Styles in general, but is a property of contexts. This is a snapshot at the location of the arc.
     std::string    number_text;     ///<The formatted number assigned to this arc (used by references and notes/comments). Set in PostParseProcess()
     std::string    wildcard_repl;   ///<What to replace \* escapes in labels.
-
-    mutable Block label_block;             ///<The calculated location of the label (if any)
+    bool delay_label_drawing = false; ///<True if our label shall be drawn after the block (used by joins with fills)
+    mutable Block label_block;      ///<The calculated location of the label (if any). For label_rotate = 90 and 180 degrees, this Block is already rotated by 90 or 270 degrees. For other angles, it needs to be rotated around its center point.
+    mutable double label_rotate = 0; ///<The rotation of the label (if any) clockwise in degrees.
 
     explicit BlockElement(BlockChart &chart);
     BlockElement(BlockChart &chart, const FileLineColRange &l);
@@ -126,7 +127,7 @@ public:
     void CollectIsMapElements(Canvas &) override;
     void RegisterLabels() override;
     Contour LabelCover() const;
-    void DrawLabel(Canvas &canvas) const;
+    void DrawLabel(Canvas &canvas, bool even_if_delayed=false) const;
     virtual void Draw(Canvas&) const {}
     ~BlockElement() override { RemoveFromDrawOrder(); }
 };
@@ -249,7 +250,6 @@ struct Constraint
 constexpr Constraint::Side SSide(unsigned v, double offset = 0) noexcept { return Constraint::Side(v, 0, 1, 0, offset); }
 constexpr Constraint::Side SSide(const VariableOffset &vo) noexcept { return SSide(vo.var, vo.offset); }
 constexpr Constraint::Side SumSide(unsigned v1, unsigned v2, double offset = 0) noexcept { return Constraint::Side(v1, v2, 1, 1, offset); }
-constexpr Constraint::Side AvgSide(unsigned v1, unsigned v2, double offset = 0) noexcept { return Constraint::Side(v1, v2, 1, 1, offset); }
 constexpr Constraint::Side WSumSide(unsigned v1, unsigned v2, double w1) noexcept { return Constraint::Side(v1, v2, w1, 1-w1, 0); }
 constexpr Constraint::Side WSumSide(unsigned v1, unsigned v2, double w1, double w2, double o = 0) noexcept { return Constraint::Side(v1, v2, w1, w2, o); }
 
@@ -421,7 +421,8 @@ public:
                                         const StringWithPosList &/*except_blocks*/,
                                         const StringWithPosList &/*except_parents*/,
                                         double /*distance*/,
-                                        const StringWithPosNumberList &/*distance_per_block*/) const { return {}; }
+                                        const StringWithPosNumberList &/*distance_per_block*/,
+                                        double /*lw2*/) const { return {}; }
     virtual bool IsMyChild(const std::unordered_set<const BlockBlock*> &/*except*/) const { return false; }
     /** When we or a parent of us is made a template, here can we unregister any drawing
     * or spacing.*/
@@ -517,10 +518,11 @@ protected:
                      bool no_offset_for_minor, bool no_offset_for_center);
     std::optional<Contour>
         GetContourForArrowsHelper(const std::vector<XY> &except, bool skip_allow_arrows,
-                                              const StringWithPosList &except_blocks,
-                                              const StringWithPosList &except_parents,
-                                              double distance,
-                                              const StringWithPosNumberList &distance_per_block) const;
+                                  const StringWithPosList& except_blocks,
+                                  const StringWithPosList& except_parents,
+                                  double distance,
+                                  const StringWithPosNumberList& distance_per_block,
+                                  double lw2) const;
 
 public:
     struct Port
@@ -532,7 +534,7 @@ public:
     const EBlockType   content_type;    ///<Same as type, except for multi-blocks which have the type of their content here
     const int          shape;           ///<If we are a shape, our number. Else -1.
     const std::string  name_original;   ///<The name of the block as specified by the user. May be empty, if the user added no name.
-    const std::string  name_full;       ///<The name of the block with its prefixes added. Empty if the user specified no name for this block (and any of the parents).
+    const std::string  name_full;       ///<The name of the block with its prefixes added. Empty if the user specified no name for this block (and any of the parents). Ends in a dot if we have no name, but our parent has. See FormChildName().
     const std::string  name_unique;     ///<The name of the block. We append the line number to make it unique.
           std::string  name_display;    ///<The human-readable name of the block. Only has line numbers for disambiguation, never empty. We disambiguate in PostParseProcess(), when we have all blocks added.
           std::string  name_gui_collapse;///<Use this name when identifying us towards the GUI for the purpose of collapse/expand. This can be the name of another entity (if we are the front element of a multi).
@@ -616,7 +618,7 @@ public:
     bool AmIContent() const override { return true; }
     void RegisterNames();
     void UnRegisterNames();
-    virtual void SetContent(BlockInstrList *c);
+    virtual void SetContent(std::variant<nullptr_t, BlockInstrList*, StringWithPos> c);
     void MakeTemplate();
     void AdjustMultiAttributes(unsigned seq, const XY &offset);
     /** Translate one 'below' 'left', etc. modifiers to an AlignmentAttr.*/
@@ -659,7 +661,8 @@ public:
                                 const StringWithPosList &except_blocks,
                                 const StringWithPosList &except_parents,
                                 double distance,
-                                const StringWithPosNumberList &distance_per_block) const override;
+                                const StringWithPosNumberList &distance_per_block, 
+                                double lw2) const override;
     Contour GetVisibleCover() const;
     bool IsMyParent(const BlockBlock *p) const { return p && (p==parent || (parent && parent->IsMyParent(p))); }
     bool IsMyChild(const std::unordered_set<const BlockBlock*> &except) const override;
@@ -677,7 +680,7 @@ public:
     XY GetLabelSize() const
     {
         XY ret = parsed_label.getTextWidthHeight();
-        if (IsHorizontal(style.label_orient.value_or(EDirection::Above)))
+        if (IsVertical(style.label_orient.value_or(EOrientation::Normal)))
             ret.SwapXY();
         return ret;
     }
@@ -743,11 +746,41 @@ public:
                         const MultiBlock* /*multi_parent*/) const override { _ASSERT(0); return nullptr; }
     bool CanBeMulti() const override { _ASSERT(0); /*parser should not get here*/ return false; }
     std::string_view CanBeCollapsed() const noexcept override { return "Indicators cannot be collapsed."; }
-    void SetContent(BlockInstrList* c) override;
+    void SetContent(std::variant<nullptr_t, BlockInstrList*, StringWithPos> c) override;
     bool PostParseProcess(Canvas& canvas, const BlockBlock* hide_by, Numbering& number) override;
     void Draw(Canvas& canvas) const override;
 };
 
+class BlockInlinedChart : public BlockBlock
+{
+protected:
+    BlockInlinedChart(const BlockInlinedChart& o, const FileLineColRange& l, std::string_view name, std::string_view prefix) :
+        BlockBlock(o, l, name, prefix), lang(o.lang), text(o.text) {
+    }
+public:
+    std::string   lang;          ///<The language name if symbol_type==INLINE
+    std::string   text;          ///<The text of the inlined chart
+    InliningData  inlining_data; ///<The compiled chart and its position
+
+
+    BlockInlinedChart(BlockChart& ch, std::string_view l, const FileLineColRange& pos, std::string_view name = {}) :
+        BlockBlock(ch, EBlockType::Chart, pos, name), lang(l) {
+    }
+    BlockBlock* CloneAs(const FileLineColRange& l, std::string_view name, std::string_view prefix,
+                        gsl::owner<AlignmentAttr*> align,
+                        gsl::owner<AttributeList*> attr, gsl::owner<CloneActionList*> mod,
+                        const MultiBlock* multi_parent) const override;
+
+    bool CanBeMulti() const override { return true; }
+    std::string_view CanBeCollapsed() const noexcept { return "Inlined charts cannot be collapsed."; }
+    void SetContent(std::variant<nullptr_t, BlockInstrList*, StringWithPos> c) override;
+    bool PostParseProcess(Canvas& canvas, const BlockBlock* hide_by, Numbering& number) override;
+    void PostPosProcess(Canvas&, Chart*) override;
+    void Draw(Canvas& canvas) const override;
+    const InliningData* GetInlinedChartData() const override { return inlining_data.chart ? &inlining_data : nullptr; }
+    void InlinedChartGUIStateChange() const override;
+};
+
 class MultiBlock : public BlockBlock
 {
     std::unique_ptr<BlockBlock> original_block; ///<The block, originally used to create us.
@@ -770,7 +803,7 @@ public:
     void ApplyRunningStyle(bool cloning, bool recursive) override;
     bool CanBeMulti() const override { _ASSERT(0); /*parser should not get here*/ return false; }
     void UpdateCollapsedFromGUI() override {} //Special handling of this in MultiBlock::SetContent()
-    void SetContent(BlockInstrList *c) override;
+    void SetContent(std::variant<nullptr_t, BlockInstrList*, StringWithPos> c) override;
 	bool PostParseProcess(Canvas &canvas, const BlockBlock* hide_by, Numbering &number) override;
 	void LayoutFinalizeBlocks(Canvas & canvas, ELayoutContentFinalizePass p) override;
 };
