@@ -164,7 +164,12 @@ extern int block_compile_debug;
     TOK_ARROW_NO = 329,            /* TOK_ARROW_NO  */
     TOK_WHITESPACE = 330,          /* TOK_WHITESPACE  */
     TOK_NEWLINE = 331,             /* TOK_NEWLINE  */
-    TOK_COMMENT = 332              /* TOK_COMMENT  */
+    TOK_COMMENT = 332,             /* TOK_COMMENT  */
+    TOK_OINLINE = 333,             /* TOK_OINLINE  */
+    TOK_CINLINE = 334,             /* TOK_CINLINE  */
+    TOK_INLINE = 335,              /* TOK_INLINE  */
+    TOK_UNMATCHED_INLINE = 336,    /* TOK_UNMATCHED_INLINE  */
+    TOK_CHART = 337                /* TOK_CHART  */
   };
   typedef enum block_compile_tokentype block_compile_token_kind_t;
 #endif
@@ -173,7 +178,7 @@ extern int block_compile_debug;
 #if ! defined BLOCK_COMPILE_STYPE && ! defined BLOCK_COMPILE_STYPE_IS_DECLARED
 union BLOCK_COMPILE_STYPE
 {
-#line 101 "./block_lang.yy"
+#line 102 "./block_lang.yy"
 
     str_view                                                   str;
     ShapeElement::Type                                         shapecommand;
@@ -211,7 +216,7 @@ union BLOCK_COMPILE_STYPE
     gsl::owner<CHAR_IF_CSH(CloneActionList)*>                  cloneactionlist;
     FIRST_IF_CSH(char*, UseCommandParseHelper)                 usecommandhelper;
 
-#line 215 "block_parser_compile.h"
+#line 220 "block_parser_compile.h"
 
 };
 typedef union BLOCK_COMPILE_STYPE BLOCK_COMPILE_STYPE;
