@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2023 Zoltan Turanyi
+    Copyright (C) 2008-2024 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -61,6 +61,17 @@ bool CaseInsensitiveEndsWith(std::string_view base, std::string_view a) noexcept
     return i2<0 || base[i2]=='.';
 }
 
+bool CaseInsensitiveSmaller(std::string_view a, std::string_view b) noexcept {
+    size_t j;
+    for (j = 0; j < a.length() && j < b.length(); j++)
+        if (toupper(a[j]) != toupper(b[j]))
+            return toupper(a[j]) < toupper(b[j]);
+    //We have reached the end of one of the strings
+    if (j == a.length())
+        return  j < b.length() || a > b; //if equal case insensitive, return case sensitive, but make 'Table' later than 'table'. Note 'Table' < 'table' in ASCII.
+    return false; //j==b.length() must hold
+}
+
 std::optional<size_t> CaseInsensitiveContains(std::string_view base, std::string_view a) noexcept {
     if (base.length() < a.length()) return {};
     for (size_t i = 0; i<=base.length()-a.length(); i++)
@@ -262,7 +273,7 @@ bool Attribute::EnsureNotClear(MscError &error, EStyleType t) const
     case EStyleType::DEF_ADD:
         return true;
     case EStyleType::ELEMENT:
-        //loss attributes are delta, so they may be unset
+        //loss attributes are delta, so they may be unset //TODO: refactor this out from here to [signalling]
         if (StartsWith("lost")) return true;
         s << "Can not unset element attribute '" << name << "'.";
         break;
