@@ -160,8 +160,10 @@ class multi_segment_string
             std::memcpy(p, a.data, a.len);
             p += a.len;
         } else if constexpr (std::is_same_v<AA, multi_segment_string>) {
-            std::memcpy(p, a.data, a.len);
-            p += a.len;
+            if (a.len) {
+                std::memcpy(p, a.data, a.len);
+                p += a.len;
+            }
             had_param |= a.had_param;
         } else if constexpr (std::is_same_v<AA, std::string_view> || std::is_same_v<AA, std::string>) {
             std::memcpy(p, a.data(), a.length());
