@@ -1,6 +1,6 @@
 /*
   This file is part of Msc-generator.
-Copyright (C) 2008-2023 Zoltan Turanyi
+Copyright (C) 2008-2024 Zoltan Turanyi
 Distributed under GNU Affero General Public License.
 
 Msc-generator is free software: you can redistribute it and/or modify
@@ -97,6 +97,8 @@ struct BackgroundFill {
     Block area;     ///<The area the background fill is applied to in chart coordinates.
 };
 
+class LanguageCollection;
+
 /** The base class for all charts.
  * It has facilities to describe the language, collect errors, shapes, chart elements
  * and their cover (for tracking), designs, supports pagination, copyright text,
@@ -129,7 +131,7 @@ public:
     virtual std::string GetLanguageDefaultText() const = 0; ///<Returns the default text of the chart in UTF-8
     virtual bool GetLanguageHasAutoheading() const = 0; ///<True if the chart has automatic headings
     virtual bool GetLanguageHasElementControls() const = 0; ///<True if the elements of the chart support GUI controls
-    virtual std::unique_ptr<Csh> CshFactory(Csh::FileListProc proc) const; ///<Return the corresponding color syntax highlighter
+    virtual std::unique_ptr<Csh> CshFactory(Csh::FileListProc proc, const LanguageCollection *languages) const; ///<Return the corresponding color syntax highlighter
     virtual std::map<std::string, std::string> RegisterLibraries() const { return {}; } ///<Return the libraries used (for the purpose of the GUI about box and commandline --version flag. In UTF-8, like "cairo v1.2.3". No need to register cairo, Msc will do that.
 
     /** @} */
@@ -142,7 +144,6 @@ public:
     /** @name Members holding chart content (entities, arcs, definitions, etc.)
     * @{ */
     MscError             Error;        ///<Collects error and warning messages
-    unsigned             current_file; ///<The number of the file under parsing. Used when generating errors.*/
     std::string          file_url;     ///<Set by the file.url chart option
     std::string          file_info;    ///<Set by the file.info chart option
     ShapeCollection      Shapes;       ///<A set of named shapes. Each one was created via 'defshape' command.
@@ -164,6 +165,21 @@ protected:
     unsigned noOverflownLabels; ///<The number of labels overflown
     /** @} */
     FillAttr GetBkFill(Range Y, bool top) const noexcept;
+
+    /** @name Members holding settings requested by the user
+    * @{ */
+    std::string forced_fontface;
+    std::string forced_fontlang;
+    FileLineColRange forced_fontlocation;
+    std::string forced_design;
+    std::optional<bool> forced_pedantic;
+    const Element* inlined_from = nullptr; ///<If this chart is inlined from another chart, this is the inlining element in the parent chart
+    /** @} */
+
+    /** Parses the input We assume that the file has already been added to Error. 
+     * 'first' char contains the position of the first character in 'input'.*/
+    virtual void ParseText(std::string_view input, const FileLineCol &first_char) = 0;
+
 public:
     /** A pointer to a procedure that reads this file and returns its content.
      * The second parameter is passed transparently.
@@ -172,11 +188,12 @@ public:
      * of the file we have read, third is the file content.*/
     typedef std::tuple<std::string, std::string, std::string> FileReadProcedure(std::string_view filename, void *param, std::string_view included_from);
 
-    Chart(FileReadProcedure *p, void *param);
+    Chart(FileReadProcedure *p, void *param, const LanguageCollection* languages);
     virtual ~Chart() = default;
-    typedef std::unique_ptr<Chart>(*ChartFactoryFunction)(FileReadProcedure *p, void *param); ///<Function type to generate a chart
+    typedef std::unique_ptr<Chart>(*ChartFactoryFunction)(FileReadProcedure *p, void *param, const LanguageCollection* languages); ///<Function type to generate a chart
     FileReadProcedure * const file_read_proc;
     void * const file_read_proc_param;
+    const LanguageCollection* const languages;
 
     /** Consumes a command-line argument specific to this chart type.
      * If the command-line argument is a general one, this function will not be called.
@@ -197,8 +214,9 @@ public:
      * For signalling charts and block diagrams, this is synonymous with
      * setting the text.font.{face,lang} chart options, for graphs it is
      * synonymous with issuing the {graph,node,edge} [ fontname = <face> ];
-     * commands (and lang is ignored).*/
-    virtual void SetDefaultFont(const std::string& face, const std::string& lang, const FileLineColRange &l) = 0;
+     * commands (and lang is ignored). If the font/face is not found or the
+     * chart does not accept this setting, we silently return.*/
+    virtual void SetDefaultFont(const std::string& face, const std::string& lang, const FileLineColRange& l) { forced_fontface = face; forced_fontlang = lang; forced_fontlocation = l; }
 
     /** Opens and loads a file for inclusion. Filename is UTF-8.
      * Returns the text to include and the position of its first character.*/
@@ -206,13 +224,13 @@ public:
 
     /** Just before compiling a chart (after filling in the designs from the designlib) apply the forced design.
      * Returns false, if design not recognized.*/
-    virtual bool ApplyForcedDesign(const string& name) = 0;
+    virtual bool ApplyForcedDesign(const string& name) { forced_design = name; return true; }
     /** Return the current state of the pedantic chart option at the outermost scope.
      * Shall be used after compilation.*/
     virtual bool GetPedantic() const noexcept = 0;
     /** Set the initial value of the pedantic chart option.
      * Shall be used before calling ParseText().*/
-    virtual void SetPedantic(bool pedantic) noexcept = 0;
+    virtual void SetPedantic(bool pedantic) noexcept { forced_pedantic = pedantic; }
     /** Returns the total bounding box of the chart (minus copyright), in chart space, not considering pagination.*/
     const Block &GetTotal() const noexcept { return total; }
     /** Returns the height of the copyright at the bottom.*/
@@ -260,6 +278,8 @@ public:
     virtual std::string SerializeGUIState() const { return string(); }
     /** Set the GUI state according to a clicked control. Returns true if a change has been made.*/
     virtual bool ControlClicked(Element *, EGUIControlType) { return false; }
+    /** Notify our parent chart (if any) when our GUI state changes.*/
+    void GUIStateChanged() { if (inlined_from) inlined_from->InlinedChartGUIStateChange(); }
     /** @} */
 
     /** @name Parse Options
@@ -282,6 +302,22 @@ public:
     std::optional<double> ParseCompassPoint(std::string_view n, const FileLineCol &l,
                                             bool report_error);
 
+    /** Compiles some in-lined text of another language.
+    * @param [in] language The language of the inlined text as defined in LanguageCollection.
+    *             May be empty, which will result in an error and empty return.
+    * @param [in] attr_lang The location of the language above. Invalid if 'language' is empty.
+    * @param [in] text The inlined chart text of 'language' language.
+    * @param [in] start The location of the first character of 'text'. File indices uses that of 'this->FileNames'.
+    * @param [in] from The Element that inlines the chart. It is stored into the returned chart.
+    * The chart text is parsed and CompleteParse() is also called, thus full layout is generated. The Errors
+    * are left in the chart, the caller can merge them to this->Error. The GUI state of the created chart is final
+    * and can be queried (or changed by ControlClicked().)
+    * @returns A chart if parsing is successful (may contain errors). The FileNames are copied from 'this', but
+    *          otherwise all locations start from line:col of 1:1.*/
+    std::unique_ptr<Chart> CompileInlineText(std::string_view language, const FileLineColRange &attr_lang,
+                                             std::string_view text, const FileLineCol& start, const Element* from,
+                                             std::string_view gui_state);
+
     /** Parse a piece of input text as a new input file.
      * This must be possible to be called several times, and must keep in-parse state
      * ready to accept new blocks of top-level syntax. (We will not break
@@ -292,7 +328,11 @@ public:
      * @param [in] input string of UTF8 encoded chart text
      * @param [in] filename The UTF8 name of the file to use in error messages.
      * @returns the number of the file as will appear in FileLineCol errors.*/
-    virtual unsigned ParseText(std::string_view input, std::string_view filename) = 0;
+    unsigned ParseText(std::string_view input, std::string_view filename) {
+        unsigned ret = Error.AddFile(filename);
+        ParseText(input, FileLineCol{(int)ret, 1, 1});
+        return ret;
+    }
 
     /** Called after all blocks of input text has been fed nto ParseText().
      * It must prepare the chart for drawing.
@@ -360,8 +400,18 @@ public:
     /** Collect info on all labels into 'labelData'. To be called during or after CompleteParse().*/
     virtual void RegisterAllLabels() = 0;
 
+    /** Draws one complete chart page or all pages.
+    * We do not draw header, footer, only the chart content.
+    * We do not fiddle with canvas preparation (assume it is prepared for our page), leave it as is.
+    * We also assume that the draw section of progress has been started and we do not end it either.
+    * We assume the page is a sane number.
+    * @param canvas The canvas to draw on
+    * @param [in] pageBreaks Governs if we draw page break indicators for multi-page charts.
+    * @param [in] page The page to draw. Zero means the whole chart. */
+    virtual void Draw(Canvas& canvas, bool pageBreaks, unsigned page) = 0;
     /** Draw the header and footer for a page.
-    * This means a potential automatic heading and the copyright text
+    * This means a potential automatic heading and the copyright text.
+    * This method expects the canvas to be 'prepared' and unprepares it.
     * @param canvas The canvas to draw on
     * @param [in] page The page to draw for. Zero means the whole chart. */
     virtual void DrawHeaderFooter(Canvas &canvas, unsigned page) = 0;
@@ -371,7 +421,7 @@ public:
     * @param canvas The canvas to draw on
     * @param [in] pageBreaks Governs if we draw page break indicators for multi-page charts.
     * @param [in] page The page to draw. Zero means the whole chart. */
-    virtual void DrawComplete(Canvas &canvas, bool pageBreaks, unsigned page) = 0;
+    virtual void DrawComplete(Canvas& canvas, bool pageBreaks, unsigned page) = 0;
 
     virtual bool DrawToFile(Canvas::EOutputType, const XY& scale,
                             std::string_view fn, bool bPageBreak, bool ignore_pagebreaks,
@@ -393,6 +443,21 @@ public:
         unsigned page, bool generateErrors = false);
     /** Clear all internal data structures and result in an empty chart as if we have parsed an empty file. */
     virtual void SetToEmpty();
+
+    /** Returns the element occupying 'xy'.
+     * If the element holds an inlined chart, we search in that, too.
+     * @param [in] xy The coordinate to look for in chart space.
+     * @returns The element found or nullptr, if none. If the element returned is in an inlined chart,
+     * we also return a transformation matrix to be inverted and applied to its areas so that they appear in the
+     * chart space of 'this'; and the beginning of the inlining (to which we need to add the 'file_pos'
+     * of the returned element.*/
+    ElementRef FindElement(const XY& xy) const;
+    /** Returns the element occupying a location in the input file.
+     * If the element holds an inlined chart, we search in that, too.
+     * @param [in] l The location in input files.
+     * @returns The element found or nullptr, if none. If the element returned is in an inlined chart,
+     *             we also return additional data. */
+    ElementRef FindElement(const FileLineCol& l) const;
 };
 
 /** A template helping Chart descendants with useful functions.
@@ -408,7 +473,7 @@ template <class Ctx = ContextBase<Style>, class Prog = ProgressBase, class RefDa
 class ChartBase : public Chart
 {
 public:
-    ChartBase(FileReadProcedure *p, void *param) : Chart(p, param) {}
+    ChartBase(FileReadProcedure *p, void *param, const LanguageCollection* languages) : Chart(p, param, languages) {}
 
     /** @name Members holding chart content (entities, arcs, definitions, etc.)
     * @{ */
@@ -458,7 +523,7 @@ public:
     /** @} */
 
     /** Sets the font in all contexts and all full designs. */
-    void SetDefaultFont(const std::string& face, const std::string& lang, const FileLineColRange& l) override;
+    void SetDefaultFont(const std::string& face, const std::string& lang, const FileLineColRange&) override;
     /** Adds an attribute list to each of a list of styles.*/
     void AddAttributeListToStyleList(const AttributeList &al, const std::vector<std::string> &styles);
     /** Adds an attribute list to each of a list of styles.*/
@@ -467,6 +532,12 @@ public:
     /** Draws page breaks (dotted or dashed lines) */
     virtual void DrawPageBreaks(Canvas &canvas);
     void DrawHeaderFooter(Canvas &canvas, unsigned page) override;
+    void DrawComplete(Canvas& canvas, bool pageBreaks, unsigned page) override {
+        //Simple default implementation. Add yours to handle progress
+        if (pageBreakData.size() == 0 || page > pageBreakData.size() || total.x.Spans() <= 0 || total.y.Spans()<=0) return;
+        Draw(canvas, pageBreaks, page);
+        DrawHeaderFooter(canvas, page);
+    }
     void SetToEmpty() override;
 };
 
@@ -605,8 +676,7 @@ inline void ChartBase<Ctx, Prog, RefData, PageData>::PopContext()
 }
 
 template<class Ctx, class Prog, class RefData, class PageData>
-inline void ChartBase<Ctx, Prog, RefData, PageData>::SetDefaultFont(const std::string& face, const std::string& lang,
-                                                                    const FileLineColRange&) {
+inline void ChartBase<Ctx, Prog, RefData, PageData>::SetDefaultFont(const std::string& face, const std::string& lang, const FileLineColRange& l) {
     //We need to set the font in the current context, but also in all full designs
     //(even if setting it) - so that if the design is used later, we have this font
     if (face.length()) {
@@ -623,6 +693,7 @@ inline void ChartBase<Ctx, Prog, RefData, PageData>::SetDefaultFont(const std::s
             if (design.IsFull())
                 design.text.lang = face;
     }
+    Chart::SetDefaultFont(face, lang, l);
 }
 
 template <class Ctx, class Prog, class RefData, class PageData>
