@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2023 Zoltan Turanyi
+    Copyright (C) 2008-2024 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -585,7 +585,7 @@ inline double Block::Distance(const Block &b) const noexcept
 /** Generic 2D Affine transformation matrix
  * The Transformation will be:
  * x' = y*ab.x + x*a1.x + a0.x;
- * y' = y*ab.y + x*a1.y + a0.y;
+ * y' = y*a1.y + x*ab.y + a0.y;
  *
  * The actual affine transformation matrix looks like
  * | a1.x ab.x a0.x |
@@ -628,7 +628,20 @@ struct TRMatrix
     constexpr bool IsDegenerate() const { return (ab.x==0 && a1.x==0) || (ab.y==0 && a1.y==0); }
     /** True if this matrix represents a transformation, that changes clockwiseness of closed contours.*/
     constexpr bool IsFlipping() const;
+    /** Multiply the two matrices, effectively resulting in applying the two transformations in sequence. */
     constexpr TRMatrix operator *(const TRMatrix &o) const;
+    /** Invert the matrix: the result will undo us. If is UB if the matrix cannot be inverted.*/
+    constexpr TRMatrix Invert() const;
+
+    /** Some 2D affine transformations can be decomposed to a combination of 
+     * shift, scale and rotate (in this order). scale can be negative.*/
+    struct Decomposed {
+        XY shift;
+        XY scale;
+        XY rotate; //x:cos, y:sin
+    };
+    /** Decompose the matrix to a rotation, scaling and shift component, if possible. Returns empty if the transformation contains sheering or negative scaling.*/
+    std::optional<Decomposed> Decompose() const;
 };
 
 /** Applies the transformation 'M' to 'xy'.*/
@@ -672,6 +685,37 @@ constexpr TRMatrix TRMatrix::operator *(const TRMatrix &o) const
                     XY(a1.x*o.a0.x + ab.x*o.a0.y + a0.x*1, ab.y*o.a0.x + a1.y*o.a0.y + a0.y*1));
 }
 
+constexpr TRMatrix TRMatrix::Invert() const {
+    const double det = a1.x * a1.y - ab.x * ab.y;
+    return TRMatrix{ //https://en.wikipedia.org/wiki/Adjugate_matrix
+        -ab/det,
+        XY{a1.y, a1.x}/det,
+        XY{ab.x * a0.y - a0.x * a1.y, -a1.x * a0.y + a0.x * ab.y}/det
+    };
+}
+
+inline std::optional<TRMatrix::Decomposed> TRMatrix::Decompose() const {
+    std::optional<TRMatrix::Decomposed> ret(std::in_place);
+    ret->shift = a0;
+    ret->scale = { sqrt(a1.x * a1.x + ab.x * ab.x), sqrt(a1.y * a1.y + ab.y * ab.y) };
+    double cos1 = a1.y / ret->scale.y;
+    double cos2 = a1.x / ret->scale.x;
+    double sin1 = -ab.x / ret->scale.x;
+    double sin2 = ab.y / ret->scale.y;
+    //if this transformation uses only rotation, positive scaling and shifting, the above values should match.
+    //we can cater for negative scaling, so allow sign differences
+    if (test_equal(fabs(cos1), fabs(cos2)) && test_equal(fabs(sin1), fabs(sin2)) && test_equal(fabs(sin1 * sin2) + fabs(cos1 * cos2), 1))
+        //try determining the sign of the scale
+        for (XY s : {XY{ 1,1 }, XY{ -1,1 }, XY{ 1,-1 }, XY{ -1,-1 }})
+            if (test_equal(s.x * cos1, s.y * cos2) && test_equal(s.x * sin1, s.y * sin2)) {
+                ret->rotate = { s.x * cos1, s.x * sin1 };
+                ret->scale.Scale(s);
+                return ret;
+            }
+    ret.reset();
+    return ret;
+}
+
 /** Returns the cosine of how many degrees shall we rotate the x axis clockwise so that it crosses 'point'.
 * All this is in a space where y grows downwards.*/
 inline double GetDegreeCos(const XY &origin, const XY &point)
