@@ -235,9 +235,21 @@ public:
     bool empty() const noexcept { return charts.empty(); }
     void clear() noexcept { zipped_content.clear(); charts.clear(); error.clear(); }
     const std::vector<PptFile::ChartInPPT> &GetCharts() const noexcept { return charts; }
+    const PptFile::ChartInPPT* FindChart(unsigned slide, unsigned chart) const noexcept {
+        const auto i = std::ranges::find_if(charts, [slide, chart](const PptFile::ChartInPPT& c) { return slide == c.slide && chart == c.chart; });
+        return i == charts.end() ? nullptr : &*i;
+    }
     std::string update(const std::vector<PptFile::ChartToPPT>& C,
                        OpenNamedFileFunction OpenNamedFile);        ///<Updates the charts. Returns any error (and leaves the state unchanged). May be called several times.
     const std::string& GetZippedContent() const noexcept { return zipped_content; } ///<Returns the current zipped content
 
     static std::string create(EmbedChartData &&data, std::string&& png_content);  //creates a fresh, zipped Art::GVML ClipFormat object to be put to the clipboard.
-};
\ No newline at end of file
+};
+
+/** Describes what we may have found in the clipboard.
+ * - monostate: nothing
+ * - PptClipboard: A fully formed Art::GVML Clipformat object, which
+ *   also contains the context of the chart (e.g., size, border, etc.)
+     and may also contain other graphics elements - even multiple charts.
+ * - EmbedChartData: Escaped text, from which we have decoded a chart. */
+using ClipboardSeen = std::variant<std::monostate, PptClipboard, EmbedChartData>;
