@@ -449,7 +449,7 @@ XY flow::FlowChart::GetKneePoint(const KneePos & k) const
 
     const bool s_fw = IsIncreasing(sdir);   //true if series dir (columns) is RIGHT or DOWN
     const Range& col_width = step->outer_edge[!dim];
-    if (k.col.side<0) ret[!dim] = s_fw ? col_width.from - arrowGap : col_width.till + arrowGap;
+    if (k.col.side<0) ret[!dim] = s_fw ? col_width.from - arrowGap : col_width.till + arrowGap; // cppcheck-suppress uninitvar
     else if (0<k.col.side) ret[!dim] = s_fw ? col_width.till + arrowGap : col_width.from - arrowGap;
     else ret[!dim] = step->center[!dim];
 
