@@ -149,8 +149,6 @@ void GraphElement::AddAttributeList(const GraphStyle *s, Chart *chart)
 
     //if the element has "label" and "original_label" attributes, we adjust them.
     auto label_w = style.write().attributes.find("label");
-    StringFormat::ExpandReferences(label_w->second.value, chart, label_w->second.linenum_value, nullptr,
-                                   false, true, StringFormat::LABEL, true);
     StringFormat basic;
     basic.Default();
     auto i = style.read().attributes.find("label_format");
@@ -159,9 +157,6 @@ void GraphElement::AddAttributeList(const GraphStyle *s, Chart *chart)
     i = style.read().attributes.find("link_format");
     if (i != style.read().attributes.end())
         basic.AddAttribute(Attribute(i->first, i->second.value, {i->second.linenum_attr, i->second.linenum_attr}, {i->second.linenum_value, i->second.linenum_value}, true), chart, EStyleType::ELEMENT);
-    //In this apply we would not recognize color or style names, hence we expanded them above
-    basic.Apply(label_w->second.value.c_str()); //do not change 'attributes["label"].value'
-    //Now expand the \c() and \s(), too, based on the basic formatting
     StringFormat::ExpandReferences(label_w->second.value, chart, label_w->second.linenum_value, &basic,
                                     false, true, StringFormat::LABEL, true);
     auto orig_w = style.write().attributes.find("original_label");
@@ -608,14 +603,14 @@ void Graph::AddToGvGraph()
         if (pNode->GetGv()==nullptr) {
             pNode->AddToGvGraph(nullptr);
             if (pNode->GetGv())
-                agbindrec(pNode->GetGv(), "Agnodeinfo_t", sizeof(Agnodeinfo_t), TRUE);	//node custom data
+                agbindrec(pNode->GetGv(), "Agnodeinfo_t", sizeof(Agnodeinfo_t), true);	//node custom data
         }
 
     /* Connect nodes */
     for (auto &pEdge : edges) {
         pEdge->AddToGvGraph();
         if (pEdge->GetGv())
-            agbindrec(pEdge->GetGv(), "Agedgeinfo_t", sizeof(Agedgeinfo_t), TRUE);	//edge custom data
+            agbindrec(pEdge->GetGv(), "Agedgeinfo_t", sizeof(Agedgeinfo_t), true);	//edge custom data
     }
 }
 
@@ -670,7 +665,7 @@ void SubGraph::AddToGvGraph() const
     //add nodes
     for (auto pNode : nodes) {
         pNode->AddToGvGraph(this);
-        agbindrec(pNode->GetGv(), "Agnodeinfo_t", sizeof(Agnodeinfo_t), TRUE);	//node custom data
+        agbindrec(pNode->GetGv(), "Agnodeinfo_t", sizeof(Agnodeinfo_t), true);	//node custom data
     }
 }
 
@@ -924,7 +919,7 @@ inline FillAttr GetFillAttr(const obj_state_t *obj)
     if (obj->fill == FILL_RADIAL)
         fill.gradient = EIntGradType::RADIAL;
     else if (obj->gradient_frac && fill.color2) {
-        fill.frac = std::max((float)0, std::min((float)1, obj->gradient_frac));
+        fill.frac = std::max(0., std::min(1., (double)obj->gradient_frac));
         fill.gradient = EIntGradType::DEGENERATE;
     }
     return fill;
