@@ -420,8 +420,8 @@ std::unique_ptr<EntityAppHelper> EntityApp::AddAttributeList(AttributeList *al,
     }
     delete ch;
 
-    //Remove children we are not defining
-    const auto bads = std::ranges::remove_if(ret->entities, [](std::unique_ptr<EntityApp>& e) {return !e->defining; });
+    //Move children we are not defining to the end
+    const auto bads = std::ranges::stable_partition(ret->entities, [](std::unique_ptr<EntityApp>& e) {return e->defining; });
     for (auto &bad : bads) {
         EntityRef ei = chart->AllEntities.Find_by_Name(bad->name);
         _ASSERT(ei);
