@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2023 Zoltan Turanyi
+    Copyright (C) 2008-2024 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -795,11 +795,17 @@ void MscContext::Empty()
                  false, false, false, false);
                  //fill line shadow only
     styles["symbol"] =
-        MscStyle(EStyleType::DEFAULT, EColorMeaning::NOHOW, EArcArrowType::NONE,
+        MscStyle(EStyleType::DEFAULT, EColorMeaning::LINE_FILL, EArcArrowType::NONE,
                  false, true, true, true, true, false,
                  false, false, false, ESideType::NO, false, false, true, false, 
                  false, false, true, false);
                  //only line fill shadow text makeroom shape
+    styles["inlined_chart"] =
+        MscStyle(EStyleType::DEFAULT, EColorMeaning::NOHOW, EArcArrowType::NONE,
+                 false, false, true, true, true, false,
+                 false, false, false, ESideType::NO, false, false, true, false,
+                 false, false, false, false);
+                 //only line fill shadow makeroom 
     styles["text"] = 
         MscStyle(EStyleType::DEFAULT, EColorMeaning::TEXT, EArcArrowType::NONE,
                  false, true, true, true, true, false,
@@ -1024,11 +1030,18 @@ void MscContext::Plain()
     p = &styles["symbol"].write();
     p->MakeCompleteButText();
     p->makeroom = true;
+
     styles["text"] = styles["symbol"];
     styles["text"].write().line.type = ELineType::NONE;
     styles["text"].write().fill.color = ColorType::none(); //fully transparent
     styles["text"].write().text += "\\mn(10)\\ms(6)"; //small font to 6, normal to 10
 
+    p = &styles["inlined_chart"].write();
+    p->MakeCompleteButText();
+    p->makeroom = true;
+    p->line.type = ELineType::NONE;
+    p->fill.color = ColorType::none();
+
     styles["note"].write().MakeCompleteButText();
     styles["note"].write().numbering.reset();
     styles["note"].write().text += "\\mn(10)\\ms(6)"; //small font to 6, normal to 10
